#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

/* 
    addcr : Routine to add a CR (\r) before the standard UNIX end-of-record
            marker, LF (\n).  Record length will be expanded by one byte to
            accommodate the new character.

    Format:
            $ addcr [-v] [input_file [output_file]]

    If no filenames are given, the standard input and output are used.  If
    one file name is given, it is assumed to be the input.  Wild cards are
    not allowed.

*/
/*  Modification History:

    12 june 1992: creation (a.c.raugh)
    21 July 1992: added error handling (ACR)
    27 sept 1993: modified messages (acr)
    21 Nove 1994: suppress status message at end when "-s" (silent mode) is
                  specified (acr)
    21 July 1995: changed -s to -v and reversed default behavior.
    15 Apr  2011: Modified called to exit() to use standard values (acr)
*/

main(argc,argv)
int argc;
char *argv[];
{
    extern int errno;   /* error code */

    FILE *infile;   /* input file           */
    FILE *outfile;  /* output file          */

    int c;       /* input character      */
    int rcount;  /* record count         */
    int maxlen;  /* maximum line length  */
    int ccount;  /* line character count */
    int verbose; /* flag value           */
    int i,j,k;   /* loop/subscript       */

/*  Open files as needed (default to standard input and output):
*/
    if (argc > 4)
      { fprintf (stderr,"Usage: addcr [-s] [input [output] ]\n");
        exit(EXIT_SUCCESS);
      }

    infile = outfile = NULL;
    verbose = 0;

    for (i=1; i<argc; ++i)
      { if (strcmp(argv[i],"-v")==0)
          { verbose = 1; }
        else if (infile == NULL)
          { if ((infile = fopen(argv[i],"r")) == NULL)
              { fprintf(stderr,"addcr: Cannot open %s for reading (%s).\n",
                                argv[i],strerror(errno));
                exit(EXIT_FAILURE);
              }
          }
        else if (outfile == NULL)
          { if ((outfile = fopen(argv[i],"w")) == NULL)
              { fprintf(stderr,"addcr: Cannot open %s for writing (%s).\n",
                                argv[i],strerror(errno));
                exit(EXIT_FAILURE);
              }
          }
      }

    infile  = ( infile==NULL)? stdin  :  infile;
    outfile = (outfile==NULL)? stdout : outfile;


/*  Loop through characters in the file.  Note that no check is made to
    determine if this file already has CR/LF
*/
    rcount = maxlen = ccount = 0;
    while ((c=getc(infile)) != EOF)
      {                              /* loop through characters */
	if (c == '\n') 
          {                          /* end of record */	
	    putc('\r',outfile);
	    putc(c,outfile);
            ++rcount;
            ccount += 2;             /* one each for \r and \n */
	    if (ccount > maxlen) maxlen = ccount;
	    ccount = 0;
          }                          /* end of record */
	  else
	  {                          /* in record */
       	    putc(c,outfile);
	    ++ccount;
          }                          /* in record */
      }                              /* loop through characters */
    
/*  Report results to standard error:
*/
    if (verbose)
      { fprintf (stderr,
             "\naddcr:  %d records written; longest record %d characters\n\n",
	     rcount,maxlen);
      }

/*  Close files:
*/
    fclose(infile);
    fclose(outfile);
    return 0;
}
