/*
    Routine to check for a carriage return before each newline in the file.


    24 Jun 1994, acr.
    15 Apr 2011, acr: Modified calls to exit() to use EXIT_SUCCESS and 
                      EXIT_FAILURE rather than specific error codes.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#define MAXLENGTH    10000
#define MAXMESSAGES     10

main(argc,argv)
  int   argc;
  char *argv[];

{ char   line[MAXLENGTH]; /* input line */
  FILE  *ifp, *ofp;   /* file pointers */
  int    unmatched;   /* count of "missing" carriage returns */
  int    record;      /* record number in file */
  int    len;

  /* open files: */

  if (argc > 2)
    { if ((ofp=fopen(argv[2],"w")) == NULL)
        { fprintf (stderr,"chkdos: Unable to open %s for writing (%s)\n",
                           argv[2],strerror(errno));
          exit(EXIT_FAILURE);
        }
    }
  else
      ofp = stdout;

  if (argc > 1)
    { if ((ifp=fopen(argv[1],"r")) == NULL)
        { fprintf (stderr,"chkdos: Unable to open %s for reading (%s)\n",
                            argv[1], strerror(errno));
          exit(EXIT_FAILURE);
        }
    }
  else
      ifp = stdin;


  /* loop through input file one line at a time: */

  record = unmatched = 0;
  while (fgets(line,MAXLENGTH,ifp) != NULL)
    { len = strlen(line);
      record++;
      if (len == MAXLENGTH  &&  line[len-1] != '\n')
        { fprintf (stderr, 
                     "chkdos: Maximum line length (%d) exceeded at line %d.\n",
                      MAXLENGTH, record);
          exit(EXIT_FAILURE);
        }

      if (line[len-2] != '\r')
        { unmatched++;
          if (unmatched <= MAXMESSAGES)
            fprintf (stderr, "chkdos: Carriage return missing in record %d.\n",
                              record);
                   if (unmatched == MAXMESSAGES)
            { fprintf (stderr, "chkdos: %d missing \"\\r\" have been reported,",
                              unmatched);
              fprintf (stderr, " no more will be displayed.\n");
            }
        }
    }

  fprintf (stderr, 
            "chkdos: %d records checked, %d were missing carriage returns.\n",
            record, unmatched);

  if (ifp != stdin) fclose(ifp);
  if (ofp != stdout) fclose(ofp);
  exit(EXIT_SUCCESS);
}
