#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

/*
   CHKFIX : This routine determines the file record length, including \n, by
            searching for the first occurence of \n and counting bytes.  It
            then continues through the file to make sure every following line 
            is the same length.  It prints a report of the line lengths which 
            differ to the standard output file.

   Format:
            $ chkfix [ input file ]

   If no input file is given the standard input is used.  Wild cards are not 
   allowed.

*/
/*  Modification History:

    26 June 1992: creation (a.c.raugh)
    21 July 1992: added file error handling (acr)
    27 Sept 1993: modified messages (acr)
    15 Apr  2011: Modified exit() calls to use standard values.
*/

main(argc,argv)
int argc;
char *argv[];
{
    extern int errno;  /* error code  */

    int c;          /* character buffer                       */
    int count;      /* current line character count           */

    int rec_len;    /* first record length                    */
    int recs;       /* number of records checked              */
    int short_recs; /* number of records shorter than REC_LEN */
    int long_recs;  /* number of records longer than REC_LEN  */

    FILE *infile;   /* input file pointer                     */

/*  Check for and open input file if necessary, otherwise use stdin:
*/
     if (argc > 1)
      { if ((infile = fopen(argv[1],"r")) == NULL) 
          { fprintf(stderr," chkfix: Unable to open %s for reading (%s)\n",
                           argv[1],strerror(errno));
            exit(EXIT_SUCCESS);
          }
      }
    else
        infile = stdin;


/*  Find first record length:
*/
    count = rec_len = recs = short_recs = long_recs = 0;
    while ((c = getc(infile)) != '\n') ++rec_len;
    ++rec_len;   /* count the \n */
    ++recs;
    printf ("\nFirst record length = %d bytes.\n\n",rec_len);

/*  Check remainder of file:
*/
    while ((c=getc(infile)) != EOF)
      { ++count;

/*      Check for end of record:
*/
        if (c == '\n')
          { ++recs;
            if (count > rec_len)
              { ++long_recs;
                printf("Record %d too long: %d bytes\n",recs,count);
              }
            else if (count < rec_len)
              { ++short_recs;
                printf("Record %d too short: %d bytes\n",recs,count);
              }
            count = 0;
          }
      }

/*  Display results:
*/
    printf ("%d records read.\n",recs);
    if (short_recs == 0 && long_recs == 0)
      printf ("All records were %d bytes long.\n",rec_len);
    else if (short_recs > 0)
      printf ("%d records were less than %d bytes long\n",short_recs,rec_len);
    else if (long_recs > 0)
      printf ("%d records were more than %d bytes long\n",long_recs,rec_len);

/*  Close file and leave:
*/
    fclose(infile);
    exit(EXIT_SUCCESS);
}
