/*
    This routine looks through a file with records of length < 10,000 and finds
    those columns which always contain a blank character.  The file need not
    have fixed length records - missing characters are considered blank,
    however.

    Format

      find_blanks <input_file>


    Output is written to the standard output.


    09 Sep 1993, acr.
    31 Jul 1998, acr: increased maximum record length 
    15 Apr 2011, acr: Modified exit() calls to use standard values.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define MAX_RECORD_LENGTH  15000

main(argc,argv)
  int   argc;
  char *argv[];

  { char line[MAX_RECORD_LENGTH];   /* input line */
    int  flag[MAX_RECORD_LENGTH];   /* array of blank flags */

    long int  rcount;               /* record count */
         int  maxlength;            /* maximum record length */
         int  minlength;            /* minimum record length */
         
         int  i,j,k;                /* loop/subscript */

         FILE *fp;                  /* input file pointer */

    static char barline[] = 
{"--------------------------------------------------------------------------------"};


    /* First, open the input file: */

    if (argc != 2)
      { printf ("Usage: find_blanks <input file>\n");
        exit(EXIT_SUCCESS);
      }

    if ((fp=fopen(argv[1],"r")) == NULL)
      { printf ("find_blanks: Unable to open %s for reading.\n",argv[1]);
        exit(EXIT_FAILURE);
      }


    /* Initialize the flag array and other variables: */

    for (i=0; i<MAX_RECORD_LENGTH; ++i) flag[i]=0;
    rcount    = 0;
    maxlength = 0;
    minlength = MAX_RECORD_LENGTH;


    /* Loop through all records in the file: */

    while (fgets(line,MAX_RECORD_LENGTH,fp) != NULL)
      { ++rcount;

        /* Loop through all characters in the record: */

        i = 0;
        while (line[i] != '\n'  &&  i < MAX_RECORD_LENGTH)
          { if (line[i] != ' ') ++flag[i];
            ++i;
          }
         
        /* Check for line lengths: */

        i = (i < MAX_RECORD_LENGTH) ? i+1 : i;
        maxlength = (i > maxlength) ? i : maxlength;
        minlength = (i < minlength) ? i : minlength; 
      }


    /* Finished with the input file.  Close it: */

    fclose(fp);


    /* Display results: */

    printf ("\n%d records checked:\n",rcount);
    printf ("  Maximum length = %5d\n  Minimum length = %5d\n\n",
             maxlength, minlength);


    printf ("  Column");
    for (i=1; i<11; ++i) printf ("  %2d   ",i);
    printf ("\n");
    printf ("%78.78s",barline);
    
    for (i=0; i<maxlength; ++i)
      { if (!(i%10)) 
          { printf ("\n");
            printf (" %6d: ",(i/10)*10);
          }
        printf ("%6d ",flag[i]);
      }
    printf ("\n");

    exit(EXIT_SUCCESS);
}
