#!/usr/bin/perl

$DEFAULT_CC = "\n";     # "\r" = Mac;  "\r\n" = DOS;  "\n" = Unix

# Routine to turn whatever carriage control is found into the carriage
# control desired.  No error checking is done - all CR and LF characters
# are subject to change.
#
# Format:  % fixcc [-m | -u | -d] [ in_file [ out_file ] ]
#
#  Standard I/O is used unless file names are supplied.  The default 
#  format desired is set above.  Otherwise:
#
#      -m  Mac 
#      -d  DOS
#      -u  Unix
#
# 22 Apr 2005,  acr
# 09 Feb 2010, ACR: Fixed stupid print statement error that finally started
#                   causing problems.
#
#============================================================================

use Getopt::Std;

if (@ARGV > 3)
  { die "Usage: fixcc [-m | -u | -d] [ in_file [ out_file ] ]\n"; }

getopts('dmu');

$infile  = (@ARGV>0)? $ARGV[0] : "-";
$outfile = (@ARGV>1)? $ARGV[1] : "-";

open (IN,$infile) || die "Could not open $file for input ($!)";
open (OUT,">$outfile") || die "Could not open $file for output ($!)";

undef $cc;

if ($opt_d)
  { $cc = "\r\n"; }

if ($opt_m)
  { if ($cc)
      { die "Usage: fixcc [-m | -u | -d] [ in_file [ out_file ] ]\n"; }
     $cc = "\r";
  }

if ($opt_u)
  { if ($cc)
      { die "Usage: fixcc [-m | -u | -d] [ in_file [ out_file ] ]\n"; }
     $cc = "\n";
  }

if (!$cc)
  { $cc = $DEFAULT_CC; }

# Read the file and convert any and all CC characters found:

while ($file = <IN>)
  { $file =~ s/(\r\n|\r|\n)/$cc/mg; 
    print OUT $file;
  }

close(OUT);
close(IN);
