/*
  Routine to divide the records of a single file into two separate files 
  (without duplication).


   Format:

     % halve [-s] [-v] count input_file output_file1 output_file2

     If "-s" is specified, then every "count" records, starting with the
     first, are extracted.  If "-v" is given, the program reports results.

   25 Oct 1994, acr.
   16 Jul 1998, acr: Removed line length restrictions
   15 Apr 2011, acr: Modified exit() calls to use standard values.

*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>


main(argc,argv)
  int    argc;
  char  *argv[];

{ int  c;           /* input character */
  FILE *ifp, *ofp1, *ofp2;  /* file pointers */
  int  count;       /* input count  */
  int  i,j,k;       /* loop/subscript */
  int  sample;      /* true if "-s" was used */
  int  verbose;     /* true if "-v" was used */
  char *rem;        /* error check for number conversion */
  char *buff;       /* holding place for string pointer */
  int  inrec, outrec1, outrec2; /* record counters */
  char infile[200], out1[200], out2[200]; /* file name holders */

  if (argc < 5 || argc > 7)
    { printf ("Usage: halve [-s] [-v] <count> <input> <output1> <output2>\n");
      exit(EXIT_SUCCESS);
    }

  /* processing arguments: */

  i=1;
  verbose = sample = 0;

  while (argv[i][0] == '-')
    { if (strcmp(argv[i],"-s") == 0) 
        { sample = 1; }
      else if (strcmp(argv[i],"-v") == 0)
        { verbose = 1; }
      else
        { printf ("halve: Invalid record count (\"%s\").\n", argv[i]);
          exit(EXIT_FAILURE);
        }
      if (++i > 3)
        { printf ("halve: Invalid record count (\"%s\").\n", argv[i]);
          exit(EXIT_FAILURE);
        }
    }

  count = strtol(argv[i], &rem, 10);
  if (strcmp(rem,"") ||  count < 1 )
    { printf ("halve: Invalid record count (\"%s\").\n",argv[i]);
      exit(EXIT_FAILURE);
    }

  if ((ifp=fopen(argv[i+1],"r")) == NULL)
    { printf ("Unable to open input file \"%s\" (%s).\n", argv[i],
                strerror(errno));
      exit(EXIT_FAILURE);
    }
  strcpy(infile,argv[i+1]);

  if ((ofp1=fopen(argv[i+2],"w")) == NULL)
    { printf ("Unable to open first output file \"%s\" (%s).\n", argv[i],
                strerror(errno));
      exit(EXIT_FAILURE);
    }
  strcpy(out1,argv[i+2]);

  if ((ofp2=fopen(argv[i+3],"w")) == NULL)
    { printf ("Unable to open second output file \"%s\" (%s).\n", argv[i],
                strerror(errno));
      exit(EXIT_FAILURE);
    }
  strcpy(out2,argv[i+3]);


  /* Set up counters and loop through the input records until end of file is
     encountered:
  */

  inrec = outrec1 = outrec2 = 0;
  c = fgetc(ifp);
  while (c != EOF)
    { /* If appropriate, we write this record to the first output file: */

      if ((sample   &&  !(inrec%count)) ||
          (!sample  &&  inrec < count))
        { outrec1++;
          while (c != '\n')
            { fputc(c,ofp1);
              c = fgetc(ifp);
            }
          fputc('\n',ofp1);
          c = fgetc(ifp);
        }

      /* Otherwise, it goes to the second output file: */

      else
        { outrec2++;
          while (c != '\n')
            { fputc(c,ofp2);
              c = fgetc(ifp);
            }
          fputc('\n',ofp2);
          c = fgetc(ifp);
        }

      inrec++;
    }

   fclose(ifp);
   fclose(ofp1);
   fclose(ofp2);

  /* print results if requested: */

  if (verbose)
    { fprintf(stderr,"halve: %6d records read from \"%s\";\n", inrec,infile);
      fprintf(stderr,"       %6d records written to \"%s\",\n",outrec1,out1);
      fprintf(stderr,"       %6d records written to \"%s\".\n",outrec2,out2);
    }

  exit(EXIT_SUCCESS);

}
