#!/usr/bin/perl

use Getopt::Std;

# Routine to force all filenames to lower case.
#
#  Format:  % lcfn [-r] [-f] [-a] <file list>
#
#   where  -r  indicates lcfn should recurse down through subdirectories
#          -f  forces overwriting of existing files without prmpting
#          -a  includes hidden files
#
# 07 Mar 2002, acr: based on old C routine
# 26 Sep 2006, acr: Modified to use Getopt::Std
#
#==========================================================================

my (@dlist,$dl,@flist,$fl);

#
#  Get the arguments and check for a file list:

getopts('afr');

if (@ARGV == 0)
  { die "Usage: lcfn [-r] [-f] [-a] <file list>\n"; }

# We loop through the list, calling the main (recursive) subroutine to 
# handle each entry:

$dl = 0;
$fl = 0;
foreach $file (@ARGV)
  { if ($file =~ /\//)
      { printf "lcfn: Error - no directories in input file names ($file).";
        printf "  Ignored.";
      }
    elsif (-d $file)
      { $dlist[$dl] = $file; 
        $dl++;
      }
    else
      { $flist[$fl] = $file; 
        $fl++;
      }
  }

# Loop through directories:

foreach $file (@dlist)
  { &dodir($file); }

# Loop through files:

foreach $file (@flist)
  { &dofile($file); }

# And we're done.

#_____________________________________________________________________________
#_____________________________________________________________________________

sub dofile

# Routine to force a file name to lowercase, checking first for conflicts with
# existing file names.

{ local ($FILE) = $_[0];

  my ($file,$answer);

  # If this is a hidden file, we ignore it unless we've been explicitly told
  # otherwise:

  return if ($FILE =~ /^\./  &&  !($opt_a));

  # Force the filename to lowercase and check for a collision:

  $file = $FILE;
  $file =~ tr/A-Z/a-z/;

  # If the file names are identical, we're done:

  return $file if ($file eq $FILE);

  if (-e $file  &&  !($opt_f))
    { printf "File '$file' already exists. Overwrite it [Y/N]? \n";
      $answer = <STDIN>;
      if ($answer =~ /^(Y|y)/)
        { rename $FILE, $file; }
    }
  else
    { rename $FILE, $file; }

  # All done:

  return $file;
}

#_____________________________________________________________________________
#_____________________________________________________________________________

sub dodir

# Routine to force a directory name to uppercase and, if desired, recurse
# down through its contents.

{ local ($DIR) = $_[0];

  my ($file,$dir);
  my (@dlist,$dl,@flist,$fl);

  # Begin by calling the file routine to rename the directory itself:

  $dir = &dofile($DIR);

  # If we're not recursing, we're done:

  return if (!($opt_r));

  # Open the directory for reading and loop through the contents:

  opendir (DIR,$dir)  ||  die "Could not open $dir for reading.";
  foreach $file (readdir(DIR))
    { next if ($file eq "."  ||  $file eq ".."); 
      next if ($file =~ /^\./  &&  !($opt_a));

      $file = "$dir/$file";

      if (-d $file)
        { $dlist[$dl] = $file;
          $dl++;
        }
      else
        { $flist[$fl] = $file;
          $fl++;
        }
    }
  closedir(DIR);

  foreach $file (@flist)
    { &dofile($file); }

  foreach $file (@dlist)
    { &dodir($file); }

  return;
}

#_____________________________________________________________________________
#_____________________________________________________________________________
#_____________________________________________________________________________
