/*

   rmVMS - Remove VMS filename version numbers

   This routine removes the semicolon and version number found at the end of
   VMS file names (and sometimes carried along to other systems).  It does this
   by reading in the string of file names given as input, moving the string
   terminator to the position of the semicolon, and calling the stdio.h 
   'rename' function.

   Format:

      % rmVMS filename [ filename [ ... ] ]

   10 Jun 1996, acr: C-code written to replace Unix shell script
   26 Aug 2002, acr: Replaced call to 'gets' to stop Linux whining
   15 Apr 2011, acr: Modified exit() calls to use standard values.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>

#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>


main(argc,argv)
  int    argc;
  char  *argv[];

{ char    *p;        /* pointer into name string */
  char    *oldname;  /* old file name holder */
  int      i,j;      /* loop variables */
  int      len;      /* string length */
  int      rm;       /* flag: true if version number should be removed */
  char     yn[20];   /* User response */

  struct stat stbuf;   /* File status buffer */


  /* Make sure there is at least one filename on the command line: */

  if (argc < 2)
    { fprintf (stderr,"Usage: rmVMS filename [ filename [ ... ] ]\n");
      exit(EXIT_SUCCESS);
    }

  /* Loop through the names on the command line: */

  for (i=1; i<argc; ++i)
    { 
      /* Look for a semicolon followed by nothing but digits.  If this is
         not found, there is nothing to do and we should move on to the next
         filename:
      */

      if ((p = strrchr(argv[i],';')) == NULL)    /* No semicolon;         */
        { rm = 0; }                              /* set removal flag off. */

      else
        { len = strlen(p);

          /* Do not remove a naked semicolon */

          rm = (len > 1)? 1 : 0;

          /* Do not remove if any of the following characters are not digits: */

          for (j=1; j<len; j++)
            { if (isdigit(p[j]) == 0)
                { rm = 0;
                  break;     /* Don't need to check further once found */
                }
            }
        }

      /* Check removal flag before going on. Go back to beginning of loop for 
         rm==0:
      */

      if (rm == 0)
        { continue; }

      /* Allocate space and save the old filename: */

      oldname = (char *)malloc(len + 1);
      strcpy(oldname,argv[i]);
      
      /* Now, write terminator over semicolon: */

      p[0] = '\0';

      /* Before calling 'rename', see if a file with this name already 
         exists by calling 'stats' on it.  It should return a -1 because
         the file is not found.  If it returns without an error, query
         the user first to see if the existing file should be over-written:
      */

      if (stat(argv[i],&stbuf) != -1)
        { fprintf (stderr,"WARNING: A file called \"%s\" already exists. ",
                           argv[i]);
          fprintf (stderr," Overwrite it? [N] ");
          fgets(yn,5,stdin);
          yn[0] = tolower(yn[0]);
          if (yn[0] != 'y')
            { continue; }       /* To next filename */
        }

      /* Rename the file: */

      if (rename(oldname,argv[i]) != 0)
        { fprintf (stderr,"rmVMS: Unable to rename \"%s\" as \"%s\" (%s).\n",
                          oldname,argv[i],strerror(errno));
        }

      /* Release the old filename string space: */

      free(oldname);

    }

  /* Done */

  exit(EXIT_SUCCESS);
}
