#!/usr/bin/perl

use Getopt::Std;

# Routine to force all filenames to upper case.
#
#  Format:  % ucfn [-r] [-f] [-a] <file list>
#
#   where  -r  indicates lcfn should recurse down through subdirectories
#          -f  forces overwriting of existing files without prmpting
#          -a  includes hidden files
#
# 07 Mar 2002, acr: based on old C routine
# 26 Sep 2006, acr: Modified to use Getopt::Std
#
#==========================================================================

my (@dlist,$dl,@flist,$fl);

#
#  Get the arguments and check for a file list:

getopts('afr');

if (@ARGV == 0)
  { die "Usage: ucfn [-r] [-f] [-a] <file list>\n"; }

# We loop through the list, calling the main (recursive) subroutine to 
# handle each entry:

$dl = 0;
$fl = 0;
foreach $file (@ARGV)
  { if ($file =~ /\//)
      { printf "lcfn: Error - no directories in input file names ($file).";
        printf "  Ignored.";
      }
    elsif (-d $file)
      { $dlist[$dl] = $file; 
        $dl++;
      }
    else
      { $flist[$fl] = $file; 
        $fl++;
      }
  }

# Loop through directories:

foreach $file (@dlist)
  { &dodir($file); }

# Loop through files:

foreach $file (@flist)
  { &dofile($file); }

# And we're done.

#_____________________________________________________________________________
#_____________________________________________________________________________

sub dofile

# Routine to force a file name to lowercase, checking first for conflicts with
# existing file names.

{ local ($file) = $_[0];

  my ($FILE,$answer);

  # If this is a hidden file, we ignore it unless we've been explicitly told
  # otherwise:

  return if ($file =~ /^\./  &&  !($opt_a));

  # Force the filename to uppercase and check for a collision:

  $FILE = $file;
  $FILE =~ tr/a-z/A-Z/;

  # If there's no difference in the file name, we're done:

  return $FILE if ($file eq $FILE);

  if (-e $FILE  &&  !($opt_f))
    { printf "File '$FILE' already exists. Overwrite it [Y/N]? \n";
      $answer = <STDIN>;
      if ($answer =~ /^(Y|y)/)
        { rename $file, $FILE; }
    }
  else
    { rename $file, $FILE; }

  # All done:

  return $FILE;
}

#_____________________________________________________________________________
#_____________________________________________________________________________

sub dodir

# Routine to force a directory name to uppercase and, if desired, recurse
# down through its contents.

{ local ($dir) = $_[0];

  my ($file,$DIR);
  my (@dlist,$dl,@flist,$fl);

  # Begin by calling the file routine to rename the directory itself:

  $DIR = &dofile($dir);

  # If we're not recursing, we're done:

  return if (!($opt_r));

  # Open the directory for reading and loop through the contents:

  opendir (DIR,$DIR)  ||  die "Could not open $dir for reading.";
  foreach $file (readdir(DIR))
    { next if ($file eq "."  ||  $file eq ".."); 
      next if ($file =~ /^\./  &&  !($opt_a));

      $file = "$DIR/$file";

      if (-d $file)
        { $dlist[$dl] = $file;
          $dl++;
        }
      else
        { $flist[$fl] = $file;
          $fl++;
        }
    }
  closedir(DIR);

  foreach $file (@flist)
    { &dofile($file); }

  foreach $file (@dlist)
    { &dodir($file); }

  return;
}

#_____________________________________________________________________________
#_____________________________________________________________________________
#_____________________________________________________________________________
