/*

   varlen: Routine to convert fixed-length records into variable-length by
           removing trailing blanks.

   Format:  % varlen [-rm] [ input_file [ output_file ] ]

   09 May 1994, acr: written in C for non-Unix systems.
   08 Jun 1994, acr: corrected to stop deleting blank lines.
   19 Apr 1999, acr: added '-rmcr' option to remove CR
   15 Apr 2011, acr: Modified exit() calls to use standard values.
   19 Apr 2011, acr: Fixed stupid lack-of-initialization-for-counter error.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#define MAXLENGTH   10000     /* Maximum record length */

main (argc,argv)
  int   argc;
  char *argv[];

{ FILE    *ifp,*ofp;         /* file pointers */
  char     line[MAXLENGTH];  /* inpuut line buffer */
  int      length;           /* input line length */
  int      count;            /* record count */
  int      i,j,k;            /* loop/subscript */
  int      RMCR;             /* TRUE if CR characters should be removed */
  int      CR;               /* TRUE if file contains CRLF carriage control */

  ifp   = NULL;
  ofp   = NULL;
  RMCR  = 0;
  count = 0;

  /* Open input files as needed: */

  if (argc > 4)
    { fprintf (stderr,"Usage: varlen [-rm] [ input_file [ output_file ] ]\n");
      exit(EXIT_SUCCESS);
    }

  i = 1;
  while (i < argc)
    { if (strcmp(argv[i],"-rmcr") == 0)
        { RMCR = 1; }

      else if (ifp == NULL)
        { if ((ifp=fopen(argv[i],"r")) == NULL)
            { fprintf (stderr,"varlen: Unable to open %s for input (%s)\n",
                               argv[i],strerror(errno));
              exit(EXIT_FAILURE);
            }
        }

      else if (ofp == NULL)
        { if ((ofp=fopen(argv[i],"w")) == NULL)
            { fprintf (stderr,"varlen: Unable to open %s for output (%s)\n",
                              argv[i],strerror(errno));
              exit(EXIT_FAILURE);
            }
        }

      i++;
    }

  if (ifp == NULL) ifp = stdin;
  if (ofp == NULL) ofp = stdout;


/*----------------------------------------------------------------------------*/

  /* Loop through input lines, removing tailing blanks from each: */

  while (fgets(line,MAXLENGTH,ifp) != NULL)
    { length = strlen(line);
      ++count;

      /* Check the first line for carriage control and set CR accordingly: */

      if (count == 1)
        { if (line[length-2] == '\r')
              CR = 1;
          else
              CR = 0;
        }

      /* Starting at the end of the string, find the first non-blank.  Note
         that this procedure removes the newline, carriage-returns (if any)
         and any other non-printing space characters as well as blanks.
      */

      i = length - 1 - CR;
      while (i >= 0  &&  isspace(line[i])) --i;

      /* Add the carriage control and string terminator and write the result: */

      if (CR && !RMCR) line[++i] = '\r';
      line[++i] = '\n';
      line[++i] = '\0';
      fputs(line,ofp);
    }


  /* Close files and exit: */

  if (ifp != stdin)  fclose(ifp);
  if (ofp != stdout) fclose(ofp);
  exit(EXIT_SUCCESS);
}
